/*
 * JBoss, Home of Professional Open Source
 * Copyright 2013, Red Hat, Inc. and/or its affiliates, and individual
 * contributors by the @authors tag. See the copyright.txt in the
 * distribution for a full listing of individual contributors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jboss.as.quickstarts.datagrid.hotrod;

import java.io.Console;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import org.infinispan.client.hotrod.RemoteCache;
import org.infinispan.client.hotrod.RemoteCacheManager;

import java.util.UUID;
import java.util.concurrent.TimeUnit;

/**
 * @author Giuseppe Bonocore
 */
public class TestCase {

    private static final String JDG_SERVERS = "jdg.servers";
    private static final String PROPERTIES_FILE = "jdg.properties";
    private static final String initialPrompt = "Choose action:\n" + "============= \n" 
    		+ "a  -  add an entry\n"
            + "l  -  loop random entry\n"
            + "p  -  print all entries\n"
            + "t  -  performance test\n"
            + "b  -  performance test batch\n"
            + "r   -  read entry\n" + "q   -  quit\n";
  
    private Console con;
    private RemoteCacheManager cacheManager;
    private RemoteCache<String, Object> cache;

    public TestCase(Console con) {
        this.con = con;
        cacheManager = new RemoteCacheManager(jdgProperty(JDG_SERVERS));
        cache = cacheManager.getCache();
        con.printf("Connected to "+jdgProperty(JDG_SERVERS)+"\n");
    }

    public void addEntry() {
        @SuppressWarnings("unchecked")
        String keyName = con.readLine("Enter key name \n");
        String keyValue = con.readLine("Enter key value \n");
        cache.put(keyName, keyValue);
    }

    public void addEntryLoop() {
      	while (true) {
		try{
	        cache.put(UUID.randomUUID().toString(), UUID.randomUUID().toString());
		Thread.sleep(10);	
		con.printf(".");
		}catch(InterruptedException e){}
         }
    }
    public void performanceTest() {
    	Long numEntry = Long.parseLong(con.readLine("Enter key number \n"));
    	Long startAll = System.nanoTime();
    	Long min = Long.MAX_VALUE;
    	Long max = 0L;
    	for(Long i =1L; i< (numEntry+1);i++ ){
    		Long startCycle =System.nanoTime();
		    cache.put(UUID.randomUUID().toString(), UUID.randomUUID().toString());
		    Long elapsed = System.nanoTime() - startCycle;
		    if(elapsed < min)
		    {
		    	min=elapsed;
		    }
		    if(elapsed > max)
		    {
		    	max=elapsed;
		    }
		    System.out.print("\r ("+i+"/"+numEntry+")");
		 }
    	Long elapsedAll=System.nanoTime()-startAll;
    	Long avg = elapsedAll/numEntry;
    	System.out.format("\n Min: %,d ns Max: %,d ns Avg: %,d ns Elapsed: %,d ns",min,max,avg,elapsedAll);
    	System.out.format("\n Min: %,d ms Max: %,d ms Avg: %,d ms Elapsed: %,d ms",TimeUnit.NANOSECONDS.toMillis(min),TimeUnit.NANOSECONDS.toMillis(max),TimeUnit.NANOSECONDS.toMillis(avg),TimeUnit.NANOSECONDS.toMillis(elapsedAll));
    	System.out.format("\n Min: %,d s Max: %,d s Avg: %,d s Elapsed: %,d s",TimeUnit.NANOSECONDS.toSeconds(min),TimeUnit.NANOSECONDS.toSeconds(max),TimeUnit.NANOSECONDS.toSeconds(avg),TimeUnit.NANOSECONDS.toSeconds(elapsedAll));
    }

    public void performanceTestBatch() {
    	Long numEntry = Long.parseLong(con.readLine("Enter key number \n"));
    	Long numBatch = Long.parseLong(con.readLine("Enter batch number \n"));
    	
    	System.out.print("\n Min: ns Max: ns Avg: ns Elapsed: ns \n");
    	for(Long j =1L; j< (numBatch+1);j++ ){
    		Long startAll = System.nanoTime();
        	Long min = Long.MAX_VALUE;
        	Long max = 0L;
	    	for(Long i =1L; i< (numEntry+1);i++ ){
	    		Long startCycle =System.nanoTime();
			    cache.put(UUID.randomUUID().toString(), UUID.randomUUID().toString());
			    Long elapsed = System.nanoTime() - startCycle;
			    if(elapsed < min)
			    {
			    	min=elapsed;
			    }
			    if(elapsed > max)
			    {
			    	max=elapsed;
			    }
			    System.out.print("\r ("+i+"/"+numEntry+")");
			 }
	    	Long elapsedAll=System.nanoTime()-startAll;
	    	Long avg = elapsedAll/numEntry;
	    	System.out.format("\r %,d | %,d | %,d | %,d \n",min,max,avg,elapsedAll);
    	}	
    }
    
    public void printEntry() {
    @SuppressWarnings("unchecked")
	String keyName = con.readLine("Enter key name \n");
	con.printf(""+cache.get(keyName)+"\n");
    }
    public void printAllEntries() {
    @SuppressWarnings("unchecked")
    Map<String, Object> allEntries= cache.getBulk();
    Set<String> keys = allEntries.keySet();
    for (String key : keys) {
    	con.printf(key+" : "+ allEntries.get(key) + "\n");
	}
    
    }

    public void stop() {
        cacheManager.stop();
    }

    public static void main(String[] args) {
    	Console con = System.console();
        TestCase testCase = new TestCase(con);
        System.out.println(initialPrompt);

        while (true) {
            String action = con.readLine(">");
            if ("a".equals(action)) {
                testCase.addEntry();
            } else if ("r".equals(action)) {
                testCase.printEntry();
            } else if ("p".equals(action)) {
                testCase.printAllEntries();
            }
            else if ("l".equals(action)) {
		     testCase.addEntryLoop();
            }else if ("t".equals(action)) {
		     testCase.performanceTest();
            }else if ("b".equals(action)) {
		     testCase.performanceTestBatch();
            }
            else if ("q".equals(action)) {
                testCase.stop();
                break;
            }
        }
    }

    public static String jdgProperty(String name) {
        Properties props = new Properties();
        try {
            props.load(TestCase.class.getClassLoader().getResourceAsStream(PROPERTIES_FILE));
        } catch (IOException ioe) {
            throw new RuntimeException(ioe);
        }
        return props.getProperty(name);
    }
}
